package ms.vmedinap.tfg.gatewayserver.security;

import com.nimbusds.jose.JOSEException;
import com.nimbusds.jose.crypto.MACVerifier;
import com.nimbusds.jwt.SignedJWT;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.security.authentication.ReactiveAuthenticationManager;
import org.springframework.security.authentication.UsernamePasswordAuthenticationToken;
import org.springframework.security.core.Authentication;
import org.springframework.security.core.GrantedAuthority;
import org.springframework.security.core.authority.SimpleGrantedAuthority;
import org.springframework.stereotype.Component;
import reactor.core.publisher.Mono;

import java.security.SignatureException;
import java.text.ParseException;
import java.util.*;
import java.util.stream.Collectors;

/**
 * Trabajo: TFG
 * Título: Seguridad adicional frente a ciber ataques “Man-In-The-Middle”
 * Carrera: Grado en Ingeniería Informática
 * Área: Ingeniería del Software
 * Curso: 2021-2022
 *
 * @author Víctor Medina Pérez
 */
@Component
public class AuthenticationManagerJwt implements ReactiveAuthenticationManager {

    @Value("${oauth.jwt.key}")
    private String jwtKey;

    @Override
    public Mono<Authentication> authenticate(Authentication authentication) {
        return Mono.just(authentication.getCredentials().toString())
                .mapNotNull(token -> {
                    SignedJWT jwt = null;
                    Map<String, Object> claimsSet = null;
                    try {
                        jwt = SignedJWT.parse(token);
                        byte[] bytes = Base64.getEncoder().encode(jwtKey.getBytes());
                        boolean verification = jwt.verify(new MACVerifier(bytes));
                        if(!verification){
                            throw new SignatureException();
                        }
                        claimsSet = jwt.getJWTClaimsSet().getClaims();
                    } catch (ParseException | JOSEException | SignatureException e) {
                        e.printStackTrace();
                    }
                    return claimsSet;
                })
                .map(claims -> {
                    String username = (String) claims.get("user_name");
                    List<String> roles = (List<String>) claims.get("authorities");
                    Collection<GrantedAuthority> authorities = roles.stream()
                            .map(SimpleGrantedAuthority::new)
                            .collect(Collectors.toList());
                    return new UsernamePasswordAuthenticationToken(username, null, authorities);
                });
    }
}
