package ms.vmedinap.tfg.gatewayserver;

import com.nimbusds.jose.JOSEException;
import com.nimbusds.jose.JWSVerifier;
import com.nimbusds.jose.crypto.MACVerifier;
import com.nimbusds.jwt.JWTClaimsSet;
import com.nimbusds.jwt.SignedJWT;
import org.junit.jupiter.api.Assertions;
import org.junit.jupiter.api.Test;
import org.springframework.boot.test.context.SpringBootTest;

import java.text.ParseException;
import java.util.Base64;
import java.util.List;
import java.util.Map;

/**
 * Trabajo: TFG
 * Título: Seguridad adicional frente a ciber ataques “Man-In-The-Middle”
 * Carrera: Grado en Ingeniería Informática
 * Área: Ingeniería del Software
 * Curso: 2021-2022
 *
 * @author Víctor Medina Pérez
 */
@SpringBootTest
public class AuthenticationManagerJwtTest {

    @Test
    void shouldAuthenticate() throws ParseException, JOSEException {
        String token = getToken();
        SignedJWT jwt = SignedJWT.parse(token);
        byte[] bytes = Base64.getEncoder().encode("jwt-signature-key-tfg-vmp".getBytes());
        JWSVerifier verifier = new MACVerifier(bytes);
        boolean verification = jwt.verify(verifier);
        Assertions.assertTrue(verification);
        JWTClaimsSet claimsSet = jwt.getJWTClaimsSet();
        Assertions.assertNotNull(claimsSet);
        Map<String, Object> claims = claimsSet.getClaims();
        Assertions.assertNotNull(claims);
        Assertions.assertNotNull((String) claims.get("user_name"));
        Assertions.assertNotNull((List<String>) claims.get("authorities"));
    }

    private String getToken() {
        String token = "eyJhbGciOiJIUzI1NiIsInR5cCI6IkpXVCJ9." +
                "eyJ0YXJqZXRhcyI6W3siaWQiOjAsInRpdHVsYXIiOiJWaWN0b3IgTWVkaW5hIFBlcmV6IiwicGFuIjoiKioqKioqKioqKioqNDQ0" +
                "NCIsImV4cGlyeV9tb250aCI6IioiLCJleHBpcnlfeWVhciI6IioqKioifSx7ImlkIjoxLCJ0aXR1bGFyIjoiVmljdG9yIE1lZGlu" +
                "YSBQZXJleiIsInBhbiI6IioqKioqKioqKioqKjU1NTUiLCJleHBpcnlfbW9udGgiOiIqIiwiZXhwaXJ5X3llYXIiOiIqKioqIn0s" +
                "eyJpZCI6MiwidGl0dWxhciI6IlZpY3RvciBNZWRpbmEgUGVyZXoiLCJwYW4iOiIqKioqKioqKioqKio2NjY2IiwiZXhwaXJ5X21v" +
                "bnRoIjoiKiIsImV4cGlyeV95ZWFyIjoiKioqKiJ9XSwidXNlcl9uYW1lIjoidm1lZGluYXBAdW9jLmVkdSIsImp3ZURhdGEiOiJl" +
                "eUpyYVdRaU9pSnFkMlZEWlhKMGFXWnBZMkYwWlNJc0ltTjBlU0k2SWtwWFJTSXNJbVZ1WXlJNklrRXlOVFpIUTAwaUxDSmhiR2Np" +
                "T2lKU1UwRXRUMEZGVUMweU5UWWlmUS5FbjBQR1liUXh5Vng1bkJjdmxHQTNYZHZPMVplTTdycThVTVVWeENZcFNyRWdMOTBpdHM5" +
                "UFVicEEzSllRRGh4LUV0bl9fYXVPUDVWTFBuZG00STZZaTFGRjhBYmF0d0NndHIwSWpsOC1lcFhQMjBqcmE0WDFqUGEtLUFESnI5" +
                "MWlNZ1VrY2NmbnJXSnFQSjc5QnR5eUxpTHo4eHE4dUM5UTZIUnVVLWdYMUZ0U1NDb2FwS1NoNm1iaElIdUZzVVNjX1V1VTlaenNn" +
                "d2QyTk0xcTJXX3BnanVzYjI3Qkgwak5hT0lTMVNiWGxOcU5QYllIMndRZ1ZpQUFtdFNsV1lqQ1MzVHBodXV4MUJ4REQxZW1ucExs" +
                "blZRRWlvdDlUdVJDdGNnVHA0aHJFX0lfY2s1WE1GQjJXcm05VVRTa3ZMSnExSERJaHJBVGRucUNPLVd1RzE4NHcuUDU3U0dLT3Ru" +
                "R0VzMHRkcS5HMXNZODlQNnBvc2U1TEo1eFV3WjMxRjFFMENIakc0YW5scGFnaGF4MXpyVjEyeUNGMkYydTZiaTd6TVg4dlRiZ2lL" +
                "NUJBVkdhb0xxbWg0MUJsQVRyVGZWaVFaMExGZTJNaHNXSDVrdjNFanZxOHNkN2t3eEpSbGpOa3hPeFNEbzhTQk9WZFBkTTNFdkNH" +
                "RjNlUVRyRzN1Y250TmZZV2JlRWFPRXdNZEdwVjZpVk9PMUVreVNCTXc0RUZkS1g3Rk12Q3hmLVVXYmtocnhiaWRwMmgyOExGVDVa" +
                "dXBrQTM4Y1lGa3kzVGhQVWVUbktKWVMyeWZoQlFnYmN0ejEydXZxVWlSdndRVkdscE5kV09mSTRCQTlMdnhvTEJBTDJ2VWZ1M0k5" +
                "ajdWaVNySDktWTg3WmlGSlJ3cWs5RVJKOF9lT0lTOTlSTVBWQjNXOUpVMkZCYjA0ZnJyXzAwdnVxa1VyZ1JNTHRYQmk5TG0xQXln" +
                "U0pGZF9EdDNTa1hCOEY2SHFEZmNOY2ltR2lIcG54c2ZIZ2ZOU2pDb3Y3ZjdHRHFVU2hFLTQ4QzJ5SW1UbFl6ZHZmTW8zTDNTYWpW" +
                "RHJtTlplQkdaVUY2aEw0ZEZHaUNsaE84UWxZbG1kUk8zVFdLWGphTmQ5RFl2YmxPUXAwdHFHMkw4eEJRcmxjTFd4c1BOU3RnUUEw" +
                "RFdZcnFjeDRwUXBnT2N1MTBsaVFSVWtDY3RaVlVVMi1kOUM2bmRUbU52aWY5YTBNY1ljczB6cDVZdVprNmxLSk0xMWE2b2NWVEc4" +
                "NndrYWM3WWl3NWQ4UFpYODVjSVRGQlE1U1I4S0NfREo1X3cxN0tPZUJ3VVdiVGdldzJwbWZlQzZCbFZHblhMY0w4eERyTXlMMGZu" +
                "UFpocEVKdlFQMW10MTVjbUpPMHB1bXdjSkRMZXFldVMwZjBCemg1eXFFellqRmVfWW93YkQ5OEZsdl9CcHlsNVJJd1o0MHVVS1Bo" +
                "TUQzX0FNODJ3XzhZZW5OcmlwaS1MVkZob0pFTWFLemNOcjRweDhiNWdnTHR1MVlxZC1HRXFodW1RdENBa3Q4YWRLWWRmTkdSVnRT" +
                "MWdSaXFkVThOMDFBSU5uREhqZEF3M3Y1alNLV1RzZ0gxU040akMzZU41dlhmOXYxc2d5TU5GTkZ6M0I3Vl84dWhZNm9oT0FOeWgx" +
                "T29jQVAtSlNrRjkxTHlGVkVwSnR0UzBOZ1lQcHB0MGl5MV9aNWRmbW5jbV9uaXpMS3hCMnZlQ293Q3RnUTJySHNjdlVvVVRQRGlr" +
                "ODNNZ2dyNXBmN3owcE1yNjcxelFtcDVRSXA0M04tZjNTZmVJU2lmb0dCTVp5T1Y0bW43V29tNmxWSlFHRFNtaWt6dUFaQndwanhl" +
                "cnpadG9pbG5pN1JtdmtrbFZ2RFlKb0pzWkRnVWhEMUlJWnBBeHllT0hDQUZpX1RFejMzWHFyS3dLeEYtUUNmTWZ2bGFhbHpsTUlr" +
                "SnJoakg4a0RDbHkxSUFNQ3N4enNKc0ZySzk4dDdwdnZzYTNlNmc0enVpSHowN1c4V21ISEJweHh5aEZkdmdqaThKeF9YNW9vcVpq" +
                "TW82dzlEcXNJTHpkMkk1UDdCNC1PcUoyTnVFRFN0UFNTY1VHR1Q4YVZ1Q0d3ZllrSXJkcmdKLWZIMVMwd0t5S0tna1liSkN1dnh3" +
                "VzZNQXlmRF9vRllEa2h4cDZjYnNBUlVjSldFV3B0Z2M5RjVURV9YTGhJLS1HVDlpS05hejBjTS1BTXNaTG1GQXk1dVhGVTBJaEg4" +
                "OXBoNXphNUNad3poU0tKVld3S0lXSmdzMUx6TDN0Q1hzZ18taTEzTkpBUXFiZkNDZjdabS1WakVIa0hsN0xsWlpGa0N6UmNyeVAy" +
                "dGJvRnowWkNZdGVyOHUtcV9kQXRUUHN2dGFXcVhLSHB0clBTMVF1STVjcTBENWd5NFl5VmIxQWEteFdPSUtFcmdCajdUZUZnd1FW" +
                "d2RBVHNCa1ZPUEJqZUtLd3VRcWdiN2FkNEhIdXN3NTZKa2pEMlVObGF2bkR2OEtrQnZKNGRZTjk2MVZqc2xKZFhBLjFLVmlKSWoy" +
                "dF9ZOFhzaTFsYklKcWciLCJzY29wZSI6WyJyZWFkIiwid3JpdGUiXSwiZXhwIjoxNjUyMjU5ODc3LCJhdXRob3JpdGllcyI6WyJS" +
                "T0xFX0FETUlOIl0sImp0aSI6IjcyZjZiNWUzLWFlYTAtNGNlOS04ZjVhLTFiMTIzOTcxZjExZCIsImNsaWVudF9pZCI6InBheW1l" +
                "bnQtc2VydmljZSJ9.NsG1YtNVQ0uOTzYTTw3qhhUHhGkaqhorqKJs9VlM8Dk";
        return token;
    }
}
