package ms.vmedinap.tfg.jwegenerator.controller;

import com.nimbusds.jose.JOSEException;
import ms.vmedinap.tfg.dbcommons.models.entity.Usuario;
import ms.vmedinap.tfg.jwegenerator.models.request.JWERequest;
import ms.vmedinap.tfg.jwegenerator.models.repository.UsuarioRepository;
import ms.vmedinap.tfg.jwegenerator.services.JweService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;

import java.io.IOException;
import java.security.KeyStoreException;
import java.security.NoSuchAlgorithmException;
import java.security.cert.CertificateException;
import java.security.spec.InvalidKeySpecException;
import java.text.ParseException;

/**
 * Trabajo: TFG
 * Título: Seguridad adicional frente a ciber ataques “Man-In-The-Middle”
 * Carrera: Grado en Ingeniería Informática
 * Área: Ingeniería del Software
 * Curso: 2021-2022
 *
 * @author Víctor Medina Pérez
 */
@RestController("jwe-service")
public class JweController {

    private JweService jweService;

    private UsuarioRepository usuarioRepository;

    @Autowired
    public JweController(JweService jweService, UsuarioRepository usuarioRepository) {
        this.jweService = jweService;
        this.usuarioRepository = usuarioRepository;
    }

    @GetMapping(path = "/jweInfoByEmail")
    public ResponseEntity<String> jweInfoByEmail(@RequestParam("email") String email) {

        ResponseEntity<String> response = null;
        JWERequest request = null;
        Usuario usuario = usuarioRepository.findByEmail(email);

        if (usuario == null) {
            return new ResponseEntity<>(null, HttpStatus.NOT_FOUND);
        }

        try {
//            request = new JWERequest(usuario.getTarjetas(), usuario);
            request = new JWERequest(usuario);
            response = jweService.generateJWE(request);
        } catch (CertificateException |
                IOException | ParseException | NoSuchAlgorithmException | InvalidKeySpecException |
                KeyStoreException | JOSEException exception) {
            return new ResponseEntity<>(null, HttpStatus.EXPECTATION_FAILED);
        }

        return response;
    }
}
