package ms.vmedinap.tfg.jwegenerator.services;

import com.nimbusds.jose.JOSEException;
import com.nimbusds.jose.JWEObject;
import com.nimbusds.jwt.SignedJWT;
import ms.vmedinap.tfg.jwegenerator.models.request.JWERequest;
import org.springframework.http.ResponseEntity;

import java.io.IOException;
import java.security.KeyStoreException;
import java.security.NoSuchAlgorithmException;
import java.security.UnrecoverableKeyException;
import java.security.cert.CertificateException;
import java.security.spec.InvalidKeySpecException;
import java.text.ParseException;

/**
 * Trabajo: TFG
 * Título: Seguridad adicional frente a ciber ataques “Man-In-The-Middle”
 * Carrera: Grado en Ingeniería Informática
 * Área: Ingeniería del Software
 * Curso: 2021-2022
 *
 * @author Víctor Medina Pérez
 */
public interface JweService {

    /**
     * Desencriptador de JWE
     *
     * @param token
     * @return
     */
    JWEObject decryptJWE(String token) throws ParseException, UnrecoverableKeyException, CertificateException,
            KeyStoreException, NoSuchAlgorithmException, IOException, JOSEException;

    /**
     * Validación del JWS contenido en el JWE
     *
     * @param signedJWT
     * @return
     */
    boolean jwsValidation(SignedJWT signedJWT) throws IOException, KeyStoreException, NoSuchAlgorithmException,
            InvalidKeySpecException, JOSEException;

    /**
     * Generador de JWE
     *
     * @param jweRequest
     * @return
     */
    ResponseEntity<String> generateJWE(JWERequest jweRequest) throws IOException, ParseException, JOSEException,
            NoSuchAlgorithmException, InvalidKeySpecException, CertificateException, KeyStoreException;
}
