package ms.vmedinap.tfg.jwegenerator.services.impl;

import com.nimbusds.jose.JOSEException;
import com.nimbusds.jose.JWEObject;
import com.nimbusds.jwt.SignedJWT;
import ms.vmedinap.tfg.dbcommons.models.entity.Role;
import ms.vmedinap.tfg.dbcommons.models.entity.Tarjeta;
import ms.vmedinap.tfg.dbcommons.models.entity.Usuario;
import ms.vmedinap.tfg.jwegenerator.models.request.JWERequest;
import ms.vmedinap.tfg.jwegenerator.services.JweService;
import org.junit.jupiter.api.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.http.ResponseEntity;

import java.io.IOException;
import java.security.KeyStoreException;
import java.security.NoSuchAlgorithmException;
import java.security.UnrecoverableKeyException;
import java.security.cert.CertificateException;
import java.security.spec.InvalidKeySpecException;
import java.text.ParseException;
import java.util.List;

import static org.junit.jupiter.api.Assertions.*;

/**
 * Trabajo: TFG
 * Título: Seguridad adicional frente a ciber ataques “Man-In-The-Middle”
 * Carrera: Grado en Ingeniería Informática
 * Área: Ingeniería del Software
 * Curso: 2021-2022
 *
 * @author Víctor Medina Pérez
 */
@SpringBootTest
class JWEServiceImplTest {

    @Autowired
    private JweService jweService;

    @Test
    void generateJWE() throws CertificateException, IOException, ParseException, NoSuchAlgorithmException,
            InvalidKeySpecException, KeyStoreException, JOSEException, UnrecoverableKeyException {
        Usuario usuario = getUsuario();
        JWERequest jweRequest = new JWERequest(usuario);
        ResponseEntity<String> response = jweService.generateJWE(jweRequest);
        assertNotNull(response);
        assertNotNull(response.getBody());
        assertNotNull(response.getStatusCode());
        JWEObject jweObject = jweService.decryptJWE(response.getBody());
        assertNotNull(jweObject);
        SignedJWT signedJWT = jweObject.getPayload().toSignedJWT();
        assertTrue(jweService.jwsValidation(signedJWT));
        String jweToString = signedJWT.getPayload().toString();
        assertNotNull(jweToString);
    }

    private Usuario getUsuario() {
        Usuario usuario = new Usuario();
        usuario.setId("USER-1");
        usuario.setNombre("Víctor");
        usuario.setApellido("Medina");
        usuario.setEmail("vmedinap@uoc.edu");
        usuario.setEnabled(true);
        usuario.setPassword("$2a$10$59gEyVnZsWdfXFPqJQxBAONRGEZ3sJGJc7jDzgaTsmc879X/aDP0S");
        usuario.setTarjetas(getTarjetas());
        usuario.setRoles(getRoles());
        return usuario;
    }

    private List<Tarjeta> getTarjetas() {
        Tarjeta tarjeta1 = new Tarjeta();
        tarjeta1.setId(1L);
        tarjeta1.setPan("1111222233334444");
        tarjeta1.setExpiry_month("1");
        tarjeta1.setExpiry_year("2040");
        tarjeta1.setTitular("Víctor Medina Pérez");
        Tarjeta tarjeta2 = new Tarjeta();
        tarjeta2.setId(2L);
        tarjeta2.setPan("2222333344445555");
        tarjeta2.setExpiry_month("2");
        tarjeta2.setExpiry_year("2040");
        tarjeta2.setTitular("Víctor Medina Pérez");
        Tarjeta tarjeta3 = new Tarjeta();
        tarjeta3.setId(2L);
        tarjeta3.setPan("3333444455556666");
        tarjeta3.setExpiry_month("3");
        tarjeta3.setExpiry_year("2040");
        tarjeta3.setTitular("Víctor Medina Pérez");
        return List.of(tarjeta1, tarjeta2, tarjeta3);
    }

    private List<Role> getRoles() {
        Role role1 = new Role();
        role1.setId("ADMIN");
        role1.setNombre("ROLE_ADMIN");
        Role role2 = new Role();
        role2.setId("USER");
        role2.setNombre("ROLE_USER");
        return List.of(role1, role2);
    }
}