package ms.vmedinap.tfg.oauthserver.service.impl;

import ms.vmedinap.tfg.dbcommons.models.entity.Usuario;
import ms.vmedinap.tfg.oauthserver.feign.JweGeneratorClient;
import ms.vmedinap.tfg.oauthserver.service.UserService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.ResponseEntity;
import org.springframework.security.core.GrantedAuthority;
import org.springframework.security.core.authority.SimpleGrantedAuthority;
import org.springframework.security.core.userdetails.User;
import org.springframework.security.core.userdetails.UserDetails;
import org.springframework.security.core.userdetails.UserDetailsService;
import org.springframework.security.core.userdetails.UsernameNotFoundException;
import org.springframework.stereotype.Service;

import java.util.List;
import java.util.stream.Collectors;

/**
 * Trabajo: TFG
 * Título: Seguridad adicional frente a ciber ataques “Man-In-The-Middle”
 * Carrera: Grado en Ingeniería Informática
 * Área: Ingeniería del Software
 * Curso: 2021-2022
 *
 * @author Víctor Medina Pérez
 */
@Service
public class UserServiceImpl implements UserService, UserDetailsService {

    @Autowired
    private JweGeneratorClient jweGeneratorClient;

    @Override
    public UserDetails loadUserByUsername(String username) throws UsernameNotFoundException {
        Usuario usuario = jweGeneratorClient.findByEmail(username);

        if (usuario == null) {
            throw new UsernameNotFoundException("Usuario no existe");
        }

        List<GrantedAuthority> authorities = usuario.getRoles().stream()
                .map(role -> new SimpleGrantedAuthority(role.getNombre()))
                .collect(Collectors.toList());

        return new User(usuario.getEmail(), usuario.getPassword(), usuario.isEnabled(),
                true,
                true, true,
                authorities);
    }

    @Override
    public Usuario findByEmail(String email) {
        return jweGeneratorClient.findByEmail(email);
    }

    @Override
    public ResponseEntity<String> jweInfoByEmail(String email) {
        return jweGeneratorClient.getJweInfoByEmail(email);
    }
}
