package ms.vmedinap.tfg.oauthserver.token;

import ms.vmedinap.tfg.dbcommons.models.entity.Tarjeta;
import ms.vmedinap.tfg.dbcommons.models.entity.Usuario;
import ms.vmedinap.tfg.oauthserver.service.UserService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.ResponseEntity;
import org.springframework.security.oauth2.common.DefaultOAuth2AccessToken;
import org.springframework.security.oauth2.common.OAuth2AccessToken;
import org.springframework.security.oauth2.provider.OAuth2Authentication;
import org.springframework.security.oauth2.provider.token.TokenEnhancer;
import org.springframework.stereotype.Component;

import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * Trabajo: TFG
 * Título: Seguridad adicional frente a ciber ataques “Man-In-The-Middle”
 * Carrera: Grado en Ingeniería Informática
 * Área: Ingeniería del Software
 * Curso: 2021-2022
 *
 * @author Víctor Medina Pérez
 */
@Component
public class AdditionalUserDetails implements TokenEnhancer {


    private final UserService userService;

    @Autowired
    public AdditionalUserDetails(UserService userService) {
        this.userService = userService;
    }

    @Override
    public OAuth2AccessToken enhance(OAuth2AccessToken oAuth2AccessToken, OAuth2Authentication oAuth2Authentication) {
        Map<String, Object> additionalUserInfo = new HashMap<>();
        Usuario usuario = userService.findByEmail(oAuth2Authentication.getName());
        List<Tarjeta> tarjetas = ofuscarTarjetas(usuario.getTarjetas());
        ResponseEntity<String> jwe = userService.jweInfoByEmail(oAuth2Authentication.getName());
        additionalUserInfo.put("tarjetas", tarjetas);
        additionalUserInfo.put("jweData", jwe.getBody());
        ((DefaultOAuth2AccessToken) oAuth2AccessToken).setAdditionalInformation(additionalUserInfo);
        return oAuth2AccessToken;
    }

    private List<Tarjeta> ofuscarTarjetas(List<Tarjeta> tarjetas) {
        return tarjetas.stream().map(
                tarjeta -> new Tarjeta(tarjeta.getId(), tarjeta.getTitular(),
                        ofuscarPan(tarjeta.getPan()),
                        ofuscarExpiryMonth(tarjeta.getExpiry_month()),
                        ofuscarExpiryYear(tarjeta.getExpiry_year()))
        ).collect(Collectors.toList());
    }

    private String ofuscarExpiryYear(String expiry_year) {
        return expiry_year.replaceAll("[0-9]", "*");
    }

    private String ofuscarExpiryMonth(String expiry_month) {
        return expiry_month.replaceAll("[0-9]", "*");
    }

    private String ofuscarPan(String pan) {
        return "************" + pan.substring(12);
    }
}
