package ms.vmedinap.tfg.paymentservice.api.services.impl;

import com.fasterxml.jackson.core.JsonProcessingException;
import com.nimbusds.jose.JOSEException;
import ms.vmedinap.tfg.dbcommons.models.entity.Usuario;
import ms.vmedinap.tfg.paymentservice.api.errors.*;
import ms.vmedinap.tfg.paymentservice.api.models.request.CardRequest;
import ms.vmedinap.tfg.paymentservice.api.models.response.CardResponse;
import ms.vmedinap.tfg.paymentservice.api.services.CardService;
import ms.vmedinap.tfg.paymentservice.api.services.JweService;
import ms.vmedinap.tfg.paymentservice.mastercard.models.MockedCardCreation;
import ms.vmedinap.tfg.paymentservice.mastercard.service.MockedService;
import ms.vmedinap.tfg.paymentservice.models.entities.Card;
import ms.vmedinap.tfg.paymentservice.models.entities.Donor;
import ms.vmedinap.tfg.paymentservice.models.repository.CardRepository;
import ms.vmedinap.tfg.paymentservice.models.repository.DonorRepository;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.text.ParseException;
import java.util.Optional;
/**
 * Trabajo: TFG
 * Título: Seguridad adicional frente a ciber ataques “Man-In-The-Middle”
 * Carrera: Grado en Ingeniería Informática
 * Área: Ingeniería del Software
 * Curso: 2021-2022
 * @author Víctor Medina Pérez
 */
@Service
public class CardServiceImpl implements CardService {

    private final CardRepository cardRepository;
    private final DonorRepository donorRepository;
    private final JweService jweService;
    private final MockedService mockedService;

    @Autowired
    public CardServiceImpl(CardRepository cardRepository, DonorRepository donorRepository, JweService jweService,
                           MockedService mockedService) {
        this.cardRepository = cardRepository;
        this.donorRepository = donorRepository;
        this.jweService = jweService;
        this.mockedService = mockedService;
    }

    @Override
    public CardResponse createCard(String jweData, CardRequest request) throws ParseException, JOSEException,
            JsonProcessingException, ApiException {
        Optional<Usuario> usuario = jweService.getUserInformation(jweData);
        MockedCardCreation mockedCardCreation;
        Card card;
        CardResponse response = new CardResponse();
        if(usuario.isPresent()){
            Optional<Donor> donor = donorRepository.findByUsuarioId(usuario.get().getId());
            if(!donor.isPresent()){
                throw new DonorNotFoundException();
            }
            checkTarjeta(request, usuario);
            Optional<Card> cardBD =
                    cardRepository.findByDonorId(donor.get().getId(),
                            request.getPrimaryAccountSuffix());
            if(cardBD.isPresent()){
                throw new CardAlreadyExistsServiceException();
            }
            mockedCardCreation = mockedService.mockedCardCreation(request.getPrimaryAccountSuffix());
            if(mockedCardCreation == null){
                throw new CardCreationServiceException();
            }
            card = setNewCard(mockedCardCreation, donor);
            cardRepository.save(card);
            response.setCardId(card.getId());
            response.setPrimaryAccountSuffix(card.getPrimaryAccountSuffix());
        }
        return response;
    }

    private Card setNewCard(MockedCardCreation mockedCardCreation, Optional<Donor> donor) {
        Card card;
        card = new Card();
        card.setId(mockedCardCreation.getCardId());
        card.setPrimaryAccountSuffix(mockedCardCreation.getPrimaryAccountSuffix());
        card.setDonor_id(donor.get().getId());
        return card;
    }

    private void checkTarjeta(CardRequest request, Optional<Usuario> usuario) throws ApiException {
        usuario.get()
                .getTarjetas()
                .stream()
                .filter(tarjeta -> tarjeta.getPan().substring(12)
                        .equals(request.getPrimaryAccountSuffix().substring(12)))
                .findFirst().orElseThrow(TarjetaNotFoundInJweServiceException::new);
    }
}
