package ms.vmedinap.tfg.paymentservice.api.services.impl;

import com.fasterxml.jackson.core.JsonProcessingException;
import com.nimbusds.jose.JOSEException;
import ms.vmedinap.tfg.dbcommons.models.entity.Usuario;
import ms.vmedinap.tfg.paymentservice.api.errors.ApiException;
import ms.vmedinap.tfg.paymentservice.api.errors.DonorAlreadyExistsServiceException;
import ms.vmedinap.tfg.paymentservice.api.models.request.DonorRequest;
import ms.vmedinap.tfg.paymentservice.api.models.response.DonorResponse;
import ms.vmedinap.tfg.paymentservice.api.services.DonorService;
import ms.vmedinap.tfg.paymentservice.api.services.JweService;
import ms.vmedinap.tfg.paymentservice.mastercard.models.MockedDonorCreation;
import ms.vmedinap.tfg.paymentservice.mastercard.service.MockedService;
import ms.vmedinap.tfg.paymentservice.models.entities.Donor;
import ms.vmedinap.tfg.paymentservice.models.repository.DonorRepository;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.text.ParseException;
import java.util.Optional;
/**
 * Trabajo: TFG
 * Título: Seguridad adicional frente a ciber ataques “Man-In-The-Middle”
 * Carrera: Grado en Ingeniería Informática
 * Área: Ingeniería del Software
 * Curso: 2021-2022
 * @author Víctor Medina Pérez
 */
@Service
public class DonorServiceImpl implements DonorService {

    private final DonorRepository donorRepository;

    private final JweService jweService;

    private final MockedService mockedService;

    @Autowired
    public DonorServiceImpl(JweService jweService, DonorRepository donorRepository, MockedService mockedService) {
        this.donorRepository = donorRepository;
        this.jweService = jweService;
        this.mockedService = mockedService;
    }

    @Override
    public DonorResponse createDonor(String jweData, DonorRequest request) throws ParseException, JOSEException,
            JsonProcessingException, ApiException {
        Optional<Usuario> usuario = jweService.getUserInformation(jweData);
        Donor donor;
        MockedDonorCreation mockedDonorCreation;
        DonorResponse response = new DonorResponse();
        if(usuario.isPresent()){
            Optional<Donor> donorBd = donorRepository.findByUsuarioId(usuario.get().getId());
            if (donorBd.isPresent()){
                throw new DonorAlreadyExistsServiceException();
            }
            mockedDonorCreation = mockedService.mockedDonorCreation(request.getEmail());
            donor = setNewDonor(request, usuario, mockedDonorCreation);
            donorRepository.save(donor);
            response.setDonorId(donor.getId());
            response.setUserName(donor.getUsername());
        }
        return response;
    }

    private Donor setNewDonor(DonorRequest request, Optional<Usuario> usuario, MockedDonorCreation mockedDonorCreation) {
        Donor donor;
        donor = new Donor();
        donor.setId(mockedDonorCreation.getDonorId());
        donor.setUsername(mockedDonorCreation.getUserName());
        donor.setUsuario_id(usuario.get().getId());
        donor.setEmail(request.getEmail());
        donor.setAcceptedConsent(request.isAcceptedConsents());
        donor.setLocale(request.getLocale());
        donor.setName(usuario.get().getNombre()+" "+ usuario.get().getApellido());
        donor.setUsertype("ONLINE");
        return donor;
    }
}
