package ms.vmedinap.tfg.paymentservice.api.services.impl;

import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.nimbusds.jose.JOSEException;
import com.nimbusds.jose.JWEObject;
import com.nimbusds.jose.crypto.RSADecrypter;
import com.nimbusds.jose.crypto.RSASSAVerifier;
import com.nimbusds.jwt.SignedJWT;
import ms.vmedinap.tfg.dbcommons.models.entity.Usuario;
import ms.vmedinap.tfg.paymentservice.api.services.JweService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.stereotype.Service;

import java.security.PrivateKey;
import java.security.interfaces.RSAPublicKey;
import java.text.ParseException;
import java.util.Optional;
/**
 * Trabajo: TFG
 * Título: Seguridad adicional frente a ciber ataques “Man-In-The-Middle”
 * Carrera: Grado en Ingeniería Informática
 * Área: Ingeniería del Software
 * Curso: 2021-2022
 * @author Víctor Medina Pérez
 */
@Service
public class JweServiceImpl implements JweService {

    @Qualifier("rsaSSAVerifier")
    private final RSAPublicKey rsaSSAVerifier;

    @Qualifier("rsaDecrypter")
    private final PrivateKey rsaDecrypter;

    @Autowired
    public JweServiceImpl(RSAPublicKey rsaSSAVerifier, PrivateKey rsaDecrypter){
        this.rsaSSAVerifier = rsaSSAVerifier;
        this.rsaDecrypter = rsaDecrypter;
    }

    public JWEObject decryptJWE(String token) throws ParseException, JOSEException {
        JWEObject jweObject;
        jweObject = JWEObject.parse(token);
        jweObject.decrypt(new RSADecrypter(rsaDecrypter));
        return jweObject;
    }

    public boolean jwsValidation(SignedJWT signedJWT) throws JOSEException {
        return signedJWT.verify(new RSASSAVerifier(rsaSSAVerifier));
    }

    @Override
    public Optional<Usuario> getUserInformation(String jweData) throws ParseException, JOSEException {
        JWEObject jwe = decryptJWE(jweData);
        SignedJWT jws = jwe.getPayload().toSignedJWT();
        Optional<Usuario> usuario = Optional.empty();
        if (jwsValidation(jws)){
            ObjectMapper objectMapper = new ObjectMapper();
            usuario = Optional
                    .of(objectMapper.convertValue(jws.getJWTClaimsSet().getClaim("usuario"), Usuario.class));
        }
        return usuario;
    }
}
