package ms.vmedinap.tfg.paymentservice.api.services.impl;

import com.fasterxml.jackson.core.JsonProcessingException;
import com.nimbusds.jose.JOSEException;
import ms.vmedinap.tfg.dbcommons.models.entity.Usuario;
import ms.vmedinap.tfg.paymentservice.api.errors.*;
import ms.vmedinap.tfg.paymentservice.api.models.request.OneTimeRequest;
import ms.vmedinap.tfg.paymentservice.api.models.response.OneTimeResponse;
import ms.vmedinap.tfg.paymentservice.api.services.JweService;
import ms.vmedinap.tfg.paymentservice.api.services.OneTimeService;
import ms.vmedinap.tfg.paymentservice.mastercard.models.MockedOneTimeTransaction;
import ms.vmedinap.tfg.paymentservice.mastercard.service.MockedService;
import ms.vmedinap.tfg.paymentservice.models.entities.Donor;
import ms.vmedinap.tfg.paymentservice.models.entities.OneTime;
import ms.vmedinap.tfg.paymentservice.models.repository.CardRepository;
import ms.vmedinap.tfg.paymentservice.models.repository.CharityRepository;
import ms.vmedinap.tfg.paymentservice.models.repository.DonorRepository;
import ms.vmedinap.tfg.paymentservice.models.repository.OneTimeRepository;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.math.BigDecimal;
import java.text.ParseException;
import java.util.Optional;
/**
 * Trabajo: TFG
 * Título: Seguridad adicional frente a ciber ataques “Man-In-The-Middle”
 * Carrera: Grado en Ingeniería Informática
 * Área: Ingeniería del Software
 * Curso: 2021-2022
 * @author Víctor Medina Pérez
 */
@Service
public class OneTimeServiceImpl implements OneTimeService {

    private final OneTimeRepository oneTimeRepository;
    private final CardRepository cardRepository;
    private final DonorRepository donorRepository;
    private final CharityRepository charityRepository;
    private final JweService jweService;
    private final MockedService mockedService;

    @Autowired
    public OneTimeServiceImpl(OneTimeRepository oneTimeRepository, CardRepository cardRepository,
                              DonorRepository donorRepository, CharityRepository charityRepository,
                              JweService jweService, MockedService mockedService) {
        this.oneTimeRepository = oneTimeRepository;
        this.cardRepository = cardRepository;
        this.donorRepository = donorRepository;
        this.charityRepository = charityRepository;
        this.jweService = jweService;
        this.mockedService = mockedService;
    }

    @Override
    public OneTimeResponse createOneTimeDonation(String jweData, OneTimeRequest request) throws ParseException,
            JOSEException, JsonProcessingException, ApiException {
        Optional<Usuario> usuario = jweService.getUserInformation(jweData);
        OneTime oneTimeTransaction;
        MockedOneTimeTransaction mockedOneTimeTransaction;
        OneTimeResponse response = new OneTimeResponse();
        if(usuario.isPresent()){
            Optional<Donor> donor = Optional.of(donorRepository.findByUsuarioId(usuario.get().getId())
                    .orElseThrow(DonorNotFoundException::new));
            mockedOneTimeTransaction = mockedService.mockedOneTimeTransaction();
            if(mockedOneTimeTransaction == null){
                throw new OneTimeTransactionServiceException();
            }
            oneTimeTransaction = setNewOneTime(request, mockedOneTimeTransaction, donor.get());
            oneTimeRepository.save(oneTimeTransaction);
            response.setTransactionId(oneTimeTransaction.getId());
            response.setTransactionStatus(oneTimeTransaction.getTransactionStatus());
            response.setTransactionMessage(mockedOneTimeTransaction.getTransactionMessage());
        }
        return response;
    }

    private OneTime setNewOneTime(OneTimeRequest request, MockedOneTimeTransaction mockedOneTimeTransaction,
                                  Donor donor)
            throws ApiException {
        OneTime oneTimeTransaction;
        oneTimeTransaction = new OneTime();
        oneTimeTransaction.setId(mockedOneTimeTransaction.getTransactionId());
        oneTimeTransaction.setAmmount(BigDecimal.valueOf(request.getAmmount()));
        oneTimeTransaction
                .setDonor(donor);
        oneTimeTransaction
                .setCard(oneTimeTransaction.getDonor().getCards()
                        .stream()
                        .filter(card -> card.getPrimaryAccountSuffix()
                                .equals(request.getPrimaryAccountSuffix()))
                        .findFirst().orElseThrow(DonorCardNotFoundException::new));

        oneTimeTransaction.setTransactionStatus(mockedOneTimeTransaction.getTransactionStatus());
        oneTimeTransaction
                .setCharity(charityRepository.findById(request.getCharityId())
                        .orElseThrow(CharityNotFoundServiceException::new));
        return oneTimeTransaction;
    }
}