package ms.vmedinap.tfg.paymentservice.configurations;

import org.springframework.beans.factory.annotation.Value;
import org.springframework.boot.autoconfigure.EnableAutoConfiguration;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.core.io.ClassPathResource;

import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.security.*;
import java.security.cert.CertificateException;
import java.security.interfaces.RSAPublicKey;
import java.security.spec.InvalidKeySpecException;
import java.security.spec.X509EncodedKeySpec;
/**
 * Trabajo: TFG
 * Título: Seguridad adicional frente a ciber ataques “Man-In-The-Middle”
 * Carrera: Grado en Ingeniería Informática
 * Área: Ingeniería del Software
 * Curso: 2021-2022
 * @author Víctor Medina Pérez
 */
@Configuration
@EnableAutoConfiguration
public class KeysConfiguration {

    @Value("${jwe.certificate.filename}")
    private String certificateName;

    @Value("${jwe.certificate.alias}")
    private String certificateAlias;

    @Value("${jwe.certificate.pwd}")
    private String certificatePwd;

    @Value("${jwe.verifierkey.filename}")
    private String verifierKeyName;

    @Bean(name = "rsaSSAVerifier")
    public RSAPublicKey getRSASSAVerifier() throws IOException, NoSuchAlgorithmException, InvalidKeySpecException {
        InputStream stream = new ClassPathResource(verifierKeyName).getInputStream();
        byte[] keyBytes = stream.readAllBytes();

        X509EncodedKeySpec spec =
                new X509EncodedKeySpec(keyBytes);
        KeyFactory kf = KeyFactory.getInstance("RSA");
        return (RSAPublicKey) kf.generatePublic(spec);
    }

    @Bean(name = "rsaDecrypter")
    public PrivateKey getRSADecrypter() throws UnrecoverableKeyException, KeyStoreException, NoSuchAlgorithmException,
            CertificateException, IOException {
        KeyStore ks;
        InputStream stream = new ClassPathResource(certificateName).getInputStream();
        byte[] pfxCertificate = stream.readAllBytes();
        try(InputStream inputStream = new ByteArrayInputStream(pfxCertificate)){
            ks = KeyStore.getInstance("PKCS12");
            ks.load(inputStream, certificatePwd.toCharArray());
        }
        return (PrivateKey) ks.getKey(certificateAlias, certificatePwd.toCharArray());
    }
}
