package ms.vmedinap.tfg.paymentservice.controller;

import com.fasterxml.jackson.core.JsonProcessingException;
import com.nimbusds.jose.JOSEException;
import ms.vmedinap.tfg.paymentservice.api.errors.*;
import ms.vmedinap.tfg.paymentservice.api.models.request.CardRequest;
import ms.vmedinap.tfg.paymentservice.api.models.request.DonorRequest;
import ms.vmedinap.tfg.paymentservice.api.models.request.OneTimeRequest;
import ms.vmedinap.tfg.paymentservice.api.models.response.CardResponse;
import ms.vmedinap.tfg.paymentservice.api.models.response.DonorResponse;
import ms.vmedinap.tfg.paymentservice.api.models.response.OneTimeResponse;
import ms.vmedinap.tfg.paymentservice.api.services.CardService;
import ms.vmedinap.tfg.paymentservice.api.services.DonorService;
import ms.vmedinap.tfg.paymentservice.api.services.OneTimeService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.web.bind.annotation.*;

import java.text.ParseException;
/**
 * Trabajo: TFG
 * Título: Seguridad adicional frente a ciber ataques “Man-In-The-Middle”
 * Carrera: Grado en Ingeniería Informática
 * Área: Ingeniería del Software
 * Curso: 2021-2022
 * @author Víctor Medina Pérez
 */
@RestController
@RequestMapping("/payments")
public class PaymentsController {

    private final DonorService donorService;

    private final CardService cardService;

    private final OneTimeService oneTimeService;

    @Autowired
    public PaymentsController(DonorService donorService, CardService cardService, OneTimeService oneTimeService) {
        this.donorService = donorService;
        this.cardService = cardService;
        this.oneTimeService = oneTimeService;
    }

    @PostMapping("/donor")
    @ResponseStatus(HttpStatus.CREATED)
    public DonorResponse registerNewDonor(@RequestHeader(name = "X-JWE-Data") String jweData,
                                          @RequestBody DonorRequest request) throws ParseException, JOSEException,
            JsonProcessingException, ApiException {
        return donorService.createDonor(jweData,request);
    }

    @PostMapping("/card")
    @ResponseStatus(HttpStatus.CREATED)
    public CardResponse registerNewCard(
            @RequestHeader(name = "X-JWE-Data") String jweData,
            @RequestBody CardRequest request) throws ParseException, JOSEException,
            JsonProcessingException, ApiException{
        return cardService.createCard(jweData, request);
    }

    @PostMapping("/one_time")
    @ResponseStatus(HttpStatus.OK)
    public OneTimeResponse registerNewOneTimeDonation(
            @RequestHeader(name = "X-JWE-Data") String jweData,
            @RequestBody OneTimeRequest request) throws ParseException, JOSEException, JsonProcessingException,
            ApiException {
        return oneTimeService.createOneTimeDonation(jweData, request);
    }

}
